/*
 *  Copyright  2003 Matthias Gall.  All Rights Reserved.
 *
 *  Questions and comments to:
 *       <mailto:matthias@sechsta-sinn.de>
 *       <http://www.sechsta-sinn.de/>
 */

#include <SDL.h>
#include <debug/Log.h>
#include <game/GameManager.h>

// --------------------------------------------------------------------------------
CLog LOG( "ProjektE.log" );
SDL_Surface* g_pBackSurface = 0;

// --------------------------------------------------------------------------------
bool InitGraphics( bool bFullScreen )
{
	LOG << "INFO: SDL Initialisierung beginnt" << ENDL;

	if( SDL_Init( SDL_INIT_VIDEO ) < 0 )
	{
		LOG << "FEHLER: Kann SDL nicht initialisieren!" << ENDL;
		return false;
	}

	unsigned long ulFlags = SDL_HWSURFACE | SDL_DOUBLEBUF;
	if( bFullScreen )
	{
		ulFlags |= SDL_FULLSCREEN;
	}

	g_pBackSurface = SDL_SetVideoMode( 800, 600, 16,  ulFlags );
	if( 0 == g_pBackSurface )
	{
		LOG << "FEHLER: Kann den Videomodus nicht setzen!" << ENDL;
	}

	LOG << "INFO: SDL Initialisierung abgeschlossen" << ENDL;

	return true;
}

// --------------------------------------------------------------------------------
void ShutdownGraphics()
{
	SDL_Quit();
}

// --------------------------------------------------------------------------------
int main( int argc, char* argv[] )
{
	LOG << "INFO: Programm startet" << ENDL;

	if( !InitGraphics( false ) )
	{
		return 0;
	}

	GAMEMANAGER->Initialize();

	SDL_Event event;
	while( 1 )
	{
		if( SDL_PollEvent( &event ) && ( event.type == SDL_KEYDOWN ) )
		{
			break;
		}

		SDL_FillRect( g_pBackSurface, 0, SDL_MapRGB( g_pBackSurface->format, 0, 0, 0 ) );

		GAMEMANAGER->BeginFrame();
		GAMEMANAGER->Idle();
		GAMEMANAGER->EndFrame();

		SDL_Flip( g_pBackSurface );
	}

	GAMEMANAGER->Uninitialize();
	
	ShutdownGraphics();

	LOG << "INFO: Programm beendet" << ENDL;

	return 0;
}