/*
 *  Copyright  2003 Matthias Gall.  All Rights Reserved.
 *
 *  Questions and comments to:
 *       <mailto:matthias@sechsta-sinn.de>
 *       <http://www.sechsta-sinn.de/>
 */

#include <SDL.h>
#include <stdlib.h>
#include "GameManager.h"

// ---------------------------------------------------------------------------
CGameManager* CGameManager::m_pInstance = 0;

// ---------------------------------------------------------------------------
CGameManager* CGameManager::GetInstance()
{
	if (!m_pInstance)
	{
		m_pInstance = new CGameManager;
	}

	return m_pInstance;
}

// ---------------------------------------------------------------------------
void CGameManager::DestroyInstance()
{
	if (m_pInstance)
	{
		delete m_pInstance;
		m_pInstance = 0;
	}
}

// ---------------------------------------------------------------------------
CGameManager::CGameManager()
:	m_dwFrameTime( 0 ),
	m_pCurrentGameState( 0 ),
	m_eCurrentGameState( CGameStateFactory::GS_UNDEFINED )
{
	m_dwFrameStart = SDL_GetTicks();
}

// ---------------------------------------------------------------------------
CGameManager::~CGameManager()
{
	if( m_pCurrentGameState )
	{
		m_pCurrentGameState->Leave();
		delete m_pCurrentGameState;
	}
}

// ---------------------------------------------------------------------------
void CGameManager::Initialize()
{
	srand( SDL_GetTicks() );

	SetGameState( CGameStateFactory::GS_GAME );
}

// ---------------------------------------------------------------------------
void CGameManager::Uninitialize()
{
	if( m_pCurrentGameState )
	{
		m_pCurrentGameState->Leave();
		delete m_pCurrentGameState;
		m_pCurrentGameState = 0;
	}
}

// ---------------------------------------------------------------------------
void CGameManager::SetGameState( CGameStateFactory::eGameState eGS )
{
	if (eGS == m_eCurrentGameState)
	{
		return;
	}

	IGameState* pNewGameState = CGameStateFactory::Create( eGS, this );
	if (!pNewGameState)
	{
		return;
	}

	if (m_pCurrentGameState)
	{
		m_pCurrentGameState->Leave();
		delete m_pCurrentGameState;
	}

	m_pCurrentGameState = pNewGameState;
	m_pCurrentGameState->Enter();

	m_eCurrentGameState = eGS;
}

// ---------------------------------------------------------------------------
void CGameManager::BeginFrame()
{
}

// ---------------------------------------------------------------------------
void CGameManager::Idle()
{
	if (m_pCurrentGameState)
	{
		m_pCurrentGameState->Idle( m_dwFrameTime );
	}
}

// ---------------------------------------------------------------------------
void CGameManager::EndFrame()
{
	DWORD dwCurrentTime = SDL_GetTicks();

	m_dwFrameTime = dwCurrentTime - m_dwFrameStart;
	m_dwFrameStart = dwCurrentTime;
}